@@ -82,7 +82,7 @@ gotweb_reply(struct request *c, int status, const char *ctype,
 	}
 
 	csp = "Content-Security-Policy: default-src 'self'; "
-	    "script-src 'none'; object-src 'none';\r\n";
+	    "script-src 'self'; object-src 'none';\r\n";
 	if (tp_writes(c->tp, csp) == -1)
 		return -1;
 
@@ -472,8 +472,7 @@ gotweb_serve_website(struct request *c, struct website *site,
 	if (site->branch_name[0] != 0) {
 		const char *branch = site->branch_name;
 
-		if (strncmp("refs/", branch, 5) != 0) {
-			branch += 5;
+		if (strncmp("refs/heads/", branch, 11) != 0) {
 			if (asprintf(&refname, "refs/heads/%s", branch) == -1) {
 				error = got_error_from_errno("asprintf");
 				goto done;
@@ -1123,11 +1122,15 @@ gotweb_free_repo_commit(struct repo_commit *rc)
 static void
 gotweb_free_repo_dir(struct repo_dir *repo_dir)
 {
+	size_t i;
+
 	if (repo_dir != NULL) {
 		free(repo_dir->name);
 		free(repo_dir->owner);
 		free(repo_dir->description);
 		free(repo_dir->url);
+		for (i = 0; i < nitems(repo_dir->sshfp); i++)
+			free(repo_dir->sshfp[i]);
 		free(repo_dir->path);
 	}
 	free(repo_dir);
@@ -1464,6 +1467,12 @@ gotweb_render_url(struct request *c, struct gotweb_url *url)
 		sep = "&";
 	}
 
+	if (url->logout == 1) {
+		if (tp_writef(c->tp, "%slogout=1", sep) == -1)
+			return -1;
+		sep = "&";
+	}
+
 	return 0;
 }
 
@@ -1697,8 +1706,33 @@ gotweb_load_got_path(struct repo_dir **rp, const char *dir,
 		if (error)
 			goto err;
 	}
-	error = gotweb_get_clone_url(&repo_dir->url, srv, repo_dir->path,
-	    dirfd(dt));
+	if (srv->show_repo_cloneurl && repo && repo->clone_url[0] != '\0') {
+		repo_dir->url = strdup(repo->clone_url);
+		if (repo_dir->url == NULL) {
+			error = got_error_from_errno("strdup");
+			goto err;
+		}
+	} else {
+		error = gotweb_get_clone_url(&repo_dir->url, srv,
+		    repo_dir->path, dirfd(dt));
+	}
+
+	if (srv->show_repo_cloneurl && repo) {
+		size_t i;
+
+		for (i = 0; i < nitems(repo->clone_url_hostkey); i++) {
+			if (repo->clone_url_hostkey[i][0] == '\0')
+				continue;
+
+			repo_dir->sshfp[i] = strdup(repo->clone_url_hostkey[i]);
+			if (repo_dir->sshfp[i] == NULL) {
+				error = got_error_from_errno("strdup");
+				goto err;
+			}
+		}
+
+	}
+
 err:
 	free(dir_test);
 	if (dt != NULL && closedir(dt) == EOF && error == NULL)
