@@ -54,6 +54,12 @@
 #define GOTWEBD_MAXPORT		 6
 #define GOTWEBD_NUMPROC		 3
 #define GOTWEBD_SOCK_FILENO	 3
+#define GOTWEBD_MAX_SSHFP	 64
+
+#define GOTWEBD_SSHFP_ECDSA	0
+#define GOTWEBD_SSHFP_ED25519	1
+#define GOTWEBD_SSHFP_RSA	2
+#define GOTWEBD_NUM_SSHFP	3
 
 #define PROC_MAX_INSTANCES	 32
 
@@ -206,6 +212,7 @@ struct repo_dir {
 	char			*owner;
 	char			*description;
 	char			*url;
+	char			*sshfp[GOTWEBD_NUM_SSHFP];
 	time_t			 age;
 	char			*path;
 };
@@ -304,6 +311,7 @@ struct querystring {
 	int		 index_page;
 	char		 path[PATH_MAX];
 	char		 login[MAX_AUTH_COOKIE];
+	int		 logout;
 };
 
 struct gotwebd_fcgi_params {
@@ -397,6 +405,8 @@ struct gotwebd_repo {
 
 	char name[NAME_MAX];
 	char description[GOTWEBD_MAXDESCRSZ];
+	char clone_url[GOTWEBD_MAXCLONEURLSZ];
+	char clone_url_hostkey[GOTWEBD_NUM_SSHFP][GOTWEBD_MAX_SSHFP];
 
 	enum gotwebd_auth_config	auth_config;
 	struct gotwebd_access_rule_list access_rules;
@@ -422,6 +432,7 @@ struct server {
 	char		 name[GOTWEBD_MAXTEXT];
 	char		 htdocs_path[PATH_MAX];
 	char		 gotweb_url_root[MAX_DOCUMENT_URI];
+	char		 sshfp[GOTWEBD_NUM_SSHFP][GOTWEBD_MAX_SSHFP];
 
 	char		 repos_path[PATH_MAX];
 	char		 repos_url_path[MAX_DOCUMENT_URI];
@@ -500,6 +511,8 @@ struct gotwebd {
 	enum gotwebd_auth_config auth_config;
 	struct gotwebd_access_rule_list access_rules;
 
+	char sshfp[GOTWEBD_NUM_SSHFP][GOTWEBD_MAX_SSHFP];
+
 	struct mediatypes mediatypes;
 
 	int		 pack_fds[GOTWEB_PACK_NUM_TEMPFILES];
@@ -547,6 +560,7 @@ struct gotweb_url {
 	const char	*folder;
 	const char	*headref;
 	const char	*path;
+	int		 logout;
 };
 
 struct querystring_keys {
@@ -568,6 +582,7 @@ enum querystring_elements {
 	GOTWEBD_QS_INDEX_PAGE,
 	GOTWEBD_QS_PATH,
 	GOTWEBD_QS_LOGIN,
+	GOTWEBD_QS_LOGOUT,
 };
 
 extern struct gotwebd	*gotwebd_env;
