@@ -20,6 +20,8 @@ export GIT_AUTHOR_NAME="Flan Hacker"
 export GIT_AUTHOR_EMAIL="flan_hacker@openbsd.org"
 export GIT_COMMITTER_NAME="$GIT_AUTHOR_NAME"
 export GIT_COMMITTER_EMAIL="$GIT_AUTHOR_EMAIL"
+export GIT_CONFIG_GLOBAL="/dev/null"
+export GIT_CONFIG_SYSTEM="/dev/null"
 export GOT_AUTHOR="$GIT_AUTHOR_NAME <$GIT_AUTHOR_EMAIL>"
 export GOT_AUTHOR_8="flan_hac"
 export GOT_AUTHOR_11="flan_hacker"
@@ -235,9 +237,10 @@ get_blob_id()
 	repo="$1"
 	tree_path="$2"
 	filename="$3"
+	commit="${4:+-c $4}"
 
-	got tree -r $repo -i $tree_path | grep "[0-9a-f] ${filename}$" | \
-		cut -d' ' -f 1
+	got tree -r $repo $commit -i $tree_path | \
+		grep "[0-9a-f] ${filename}$" | cut -d' ' -f 1
 }
 
 test_init()
