@@ -44,7 +44,7 @@ commit $head_id (master)
 from: Flan Hacker <flan_hacker@openbsd.org>
 date: $date
 
-adding the test tree
+ adding the test tree
 
 A  alpha         |  1+  0-
 A  beta          |  1+  0-
@@ -93,7 +93,7 @@ $ymd flan_hacker  [master] adding the test tree        |commit $head_id (master)
                                                              |from: Flan Hacker <flan_hacker@openbsd.org>
                                                              |date: $date
                                                              |
-                                                             |adding the test tree
+                                                             | adding the test tree
                                                              |
                                                              |A  alpha         |  1+  0-
                                                              |A  beta          |  1+  0-
@@ -290,6 +290,49 @@ EOF
 		return 1
 	fi
 
+	# test T keymap on a work tree entry
+	got checkout "$testroot/repo" "$testroot/wt" > /dev/null
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "got checkout failed unexpectedly"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	cd "$testroot/wt"
+	echo "'alpha" >> alpha
+
+	local id10=$(trim_obj_id 10 $head_id)
+
+	cat <<-EOF >$TOG_TEST_SCRIPT
+	WAIT_FOR_UI	wait for log thread to fetch wt state
+	T		open tree view
+	S		toggle horizontal split
+	SCREENDUMP
+	EOF
+
+	cat <<EOF >$testroot/view.expected
+diff $testroot/wt (work tree changes) [0/1] maste
+$ymd flan_hacker  work tree changes based on [$id10]
+--------------------------------------------------------------------------------
+commit $head_id
+[1/4] /
+
+  alpha
+  beta
+  epsilon/
+  gamma/
+EOF
+
+	tog log
+	cmp -s "$testroot/view.expected" "$testroot/view"
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u "$testroot/view.expected" "$testroot/view"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
 	test_done "$testroot" "$ret"
 }
 
@@ -313,12 +356,14 @@ SCREENDUMP
 EOF
 
 	cat <<EOF >$testroot/view.expected
-[1/26] diff $commit1 $head_id
+[1/25] diff $commit1 $head_id
 commit $head_id (master)
 from: Flan Hacker <flan_hacker@openbsd.org>
 date: $date
 
-$(widechar_logmsg)
+ $(widechar_logmsg1)
+$(echo)
+ $(widechar_logmsg2)
 
 A  $(widechar_filename)  |  5+  0-
 
@@ -339,6 +384,7 @@ blob + $blobid (mode 644)
 
 
 
+
 (END)
 EOF
 
@@ -499,7 +545,7 @@ test_log_commit_keywords()
 test_log_show_base_commit()
 {
 	# make view wide enough to show full headline
-	test_init log_show_base_commit 80 3
+	test_init log_show_base_commit 80 4
 	local repo="$testroot/repo"
 	local id=$(git_show_head "$repo")
 
@@ -531,6 +577,7 @@ test_log_show_base_commit()
 	$(trim 80 "commit $head_id [1/2] master")
 	$ymd flan_hacker *[master] base commit
 	$ymd flan_hacker  adding the test tree
+
 	EOF
 
 	tog log
@@ -547,6 +594,7 @@ test_log_show_base_commit()
 	$(trim 80 "commit $head_id [1/2] master")
 	$ymd flan_hacker  [master] base commit
 	$ymd flan_hacker  adding the test tree
+
 	EOF
 
 	tog log -r "$repo"
@@ -572,6 +620,31 @@ test_log_show_base_commit()
 	$(trim 80 "commit $head_id [1/3] master")
 	$ymd flan_hacker ~[master] new base mixed-commit
 	$ymd flan_hacker  base commit
+	$ymd flan_hacker  adding the test tree
+	EOF
+
+	tog log
+	cmp -s "$testroot/view.expected" "$testroot/view"
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u "$testroot/view.expected" "$testroot/view"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	# check marker is not incorrectly drawn when opening a nested log view
+	cat <<-EOF >$TOG_TEST_SCRIPT
+	T		# open tree view of base commit
+	j		# select beta tree entry
+	L		# load log view of commits involving beta
+	SCREENDUMP
+	EOF
+
+	cat <<-EOF >$testroot/view.expected
+	$(trim 80 "commit $id /beta [1/1]")
+	$ymd flan_hacker  adding the test tree
+
+
 	EOF
 
 	tog log
@@ -902,6 +975,168 @@ test_log_mark_keymap()
 	test_done "$testroot" $ret
 }
 
+test_log_worktree_entries()
+{
+	test_init log_worktree_entries 96 4
+	local repo="$testroot/repo"
+
+	got checkout "$repo" "$testroot/wt" > /dev/null
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "got checkout failed unexpectedly"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	# test dirty work tree (no staged changes)
+	cd "$testroot/wt"
+	echo "'alpha" >> alpha
+
+	local id=$(git_show_head "$repo")
+	local id10=$(trim_obj_id 10 $id)
+	local author_time=$(git_show_author_time "$repo")
+	local ymd=$(date -u -r "$author_time" +"%F")
+
+	cat <<-EOF >$TOG_TEST_SCRIPT
+	WAIT_FOR_UI	wait for log thread to fetch wt state
+	WAIT_FOR_UI	trigger redraw to reveal work tree entries
+	SCREENDUMP
+	EOF
+
+	cat <<-EOF >$testroot/view.expected
+	diff $testroot/wt (work tree changes) [0/1] master
+	$ymd flan_hacker  work tree changes based on [$id10]
+	$ymd flan_hacker *[master] adding the test tree
+
+	EOF
+
+	tog log
+	cmp -s "$testroot/view.expected" "$testroot/view"
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u "$testroot/view.expected" "$testroot/view"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	# test dirty work tree with staged changes too
+	echo "'beta" >> beta
+	got stage beta > /dev/null
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "got stage failed unexpectedly"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	cat <<-EOF >$TOG_TEST_SCRIPT
+	WAIT_FOR_UI	wait for log thread to fetch wt state
+	j		select staged entry
+	SCREENDUMP
+	EOF
+
+	cat <<-EOF >$testroot/view.expected
+	diff -s $testroot/wt (staged work tree changes) [0/1] master
+	$ymd flan_hacker  work tree changes based on [$id10]
+	$ymd flan_hacker  staged work tree changes based on [$id10]
+	$ymd flan_hacker *[master] adding the test tree
+	EOF
+
+	tog log
+	cmp -s "$testroot/view.expected" "$testroot/view"
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u "$testroot/view.expected" "$testroot/view"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	# test with only staged changes
+	got revert alpha > /dev/null
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "got revert failed unexpectedly"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	cat <<-EOF >$TOG_TEST_SCRIPT
+	WAIT_FOR_UI	wait for log thread to fetch wt state
+	WAIT_FOR_UI	trigger redraw to reveal work tree entries
+	SCREENDUMP
+	EOF
+
+	cat <<-EOF >$testroot/view.expected
+	diff -s $testroot/wt (staged work tree changes) [0/1] master
+	$ymd flan_hacker  staged work tree changes based on [$id10]
+	$ymd flan_hacker *[master] adding the test tree
+
+	EOF
+
+	tog log
+	cmp -s "$testroot/view.expected" "$testroot/view"
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u "$testroot/view.expected" "$testroot/view"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	# test with no author set
+	echo "'alpha" >> alpha
+
+	cat <<-EOF >$testroot/view.expected
+	diff $testroot/wt (work tree changes) [0/1] master
+	$ymd              work tree changes based on [$id10]
+	$ymd              staged work tree changes based on [$id10]
+	$ymd flan_hacker *[master] adding the test tree
+	EOF
+
+	unset GOT_AUTHOR
+	tog log
+	cmp -s "$testroot/view.expected" "$testroot/view"
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u "$testroot/view.expected" "$testroot/view"
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	test_done "$testroot" 0
+}
+
+test_log_tiny_child_tree_view()
+{
+	test_init log_tiny_child_tree_view 120 3
+
+	local id=$(git_show_head $testroot/repo)
+
+	# This test covers the offset_selection_down() path ensuring
+	# indexes are properly clamped to prevent negative values.
+	cat <<-EOF >$TOG_TEST_SCRIPT
+	T		open tree view in vsplit
+	F		toggle fullscreen to hit offset_selection_down()
+	SCREENDUMP
+	EOF
+
+	cat <<-EOF >$testroot/view.expected
+	commit $id
+	[1/4] /
+
+	EOF
+
+	cd $testroot/repo && tog log
+	cmp -s $testroot/view.expected $testroot/view
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u $testroot/view.expected $testroot/view
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	test_done "$testroot" "$ret"
+}
+
 test_parseargs "$@"
 run_test test_log_hsplit_diff
 run_test test_log_vsplit_diff
@@ -915,3 +1150,5 @@ run_test test_log_show_base_commit
 run_test test_log_limit_view
 run_test test_log_search
 run_test test_log_mark_keymap
+run_test test_log_worktree_entries
+run_test test_log_tiny_child_tree_view
